import gomea.fitness
import gomea.real_valued
import gomea.discrete
import gomea.linkage

def RealValuedGOMEA(**kwargs):
    return gomea.real_valued.RealValuedGOMEA(**kwargs)

def DiscreteGOMEA(**kwargs):
    return gomea.discrete.DiscreteGOMEA(**kwargs)